/* mini-at/src/mini-at/stdin.c
 *
 *  (c)2006, Laurence Withers, <l@lwithers.me.uk>.
 *  Released under the GNU GPLv2. See file COPYING or
 *  http://www.gnu.org/copyleft/gpl.html for details.
*/



/* do_stdin()
 *
 *  Copies data from the file specified in `filename' (or from stdin if `-'). At present, we don't 
 *  create a copy of an existing file; we merely point at it. Perhaps an option could be added to
 *  do the copy, but `cat' could do the job just as easily.
 */
const char* do_stdin(const char* filename)
{
    ssize_t rd;
    char buf[1024];
    static char fn[] = "/tmp/mini-at-stdin.XXXXXX";
    int tfd;

    // just point at existing file
    if(strcmp(filename, "-")) return filename;

    // open temporary file (relies on libc opening it mode 0600)
    tfd = mkstemp(fn);
    if(tfd == -1) {
        perror("mkstemp");
        return 0;
    }

    // copy stdin to a temporary file
    while(1) {
        rd = read(0, buf, sizeof(buf));
        switch(rd) {
        case -1:
            perror("read");
            return 0;

        case 0:     // EOF
            return fn;

        default:
            if(write(tfd, buf, rd) != rd) {
                perror("write");
                return 0;
            }
        }
    }
}



/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
vim: expandtab:ts=4:sw=4
*/
