/* mini-at/src/mini-atd/util.c
 *
 *  (c)2006, Laurence Withers, <l@lwithers.me.uk>.
 *  Released under the GNU GPLv2. See file COPYING or
 *  http://www.gnu.org/copyleft/gpl.html for details.
*/



/* safe_malloc()
 *
 *  Allocates `n' bytes, aborting the program if we run out of memory. Always retruns a valid 
 *  pointer. Clears memory.
 */
void* safe_malloc(size_t n)
{
    void* p;
    
    p = malloc(n);
    if(!p) {
        LOG(LOG_CRIT, "error allocating %lu bytes of memory (%d - %s)", 
            (unsigned long)n, errno, strerror(errno));
        abort();
    }
    memset(p, 0, n);
    return p;
}



/* safe_realloc()
 *
 *  Reallocates `p' so that it is `n' bytes in size, aborting the program if we run out of memory.
 *  Always returns a valid pointer. Semantics as realloc(3).
 */
void* safe_realloc(void* p, size_t n)
{
    p = realloc(p, n);
    if(!p) {
        LOG(LOG_CRIT, "error reallocating %lu bytes of memory (%d - %s)",
            (unsigned long)n, errno, strerror(errno));
        abort();
    }
    return p;
}



/* safe_strdup()
 *
 *  Returns a newly-malloc()ed duplicate of `s', aborting the program if we run out of memory.
 *  Always returns a valid pointer.
 */
char* safe_strdup(const char* s)
{
    char* p;

    p = strdup(s);
    if(!p) {
        LOG(LOG_CRIT, "error allocating %lu bytes of memory (%d - %s)",
            (unsigned long)(strlen(s)), errno, strerror(errno));
        abort();
    }
    return p;
}



/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
vim: expandtab:ts=4:sw=4
*/
